/*****************************************************************************
 * Application Name: SIMG32 - DIMG 32bit Simple Imager for Windows
 *        Copyright: (C) Copyright David Goodwin, 2010
 *          License: GNU GPL v2
 *****************************************************************************
	This file is part of "WinDIMG32".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * Windows Headers
 */
#include <windows.h>
#include <commdlg.h>
#include <commctrl.h>

#include <stdio.h>

/*
 * Program Headers
 */
#include "resource.h"
#include "helpmap.h"
#include "imager.h"
#include "CmdLineToArgvA.h"

#ifdef UNICODE
#define CommandLineToArgv CommandLineToArgvW
#else
#define CommandLineToArgv CommandLineToArgvA
#endif

/*
 * Constants
 */
#define MAX_FILE_LEN	120
#define	HELP_FILE		TEXT("simg.hlp")

/* 
 * Forward Declarations
 */
LRESULT CALLBACK MainDlgProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK AboutDlgProc(HWND, UINT, WPARAM, LPARAM);
void browseForImageFile(HWND);
void clearFilenameTextBox(HWND);
void populateDriveComboBox(HWND);
void processArgs();

/*
 * Local variables
 */
static int image_mode;
static TCHAR szFilterSpec[256] = 
	TEXT("Disk Image Files (*.IMA,*.IMG)\0*.IMA;*.IMG\0All Files (*.*)\0*.*\0");
static HINSTANCE currentInstance;
static char selectedDrive;

/*
 * Local variables that can be set from the command line
 */
static TCHAR szFileName[MAX_FILE_LEN];	/* Filename */
static bool showAbout;		 /* Show about dialog on startup */
static bool setWrite;		 /* Make write operation default */
//static bool setRead;	// default
static int setDrive;		 /* Default drive */
static bool justGo;			 /* Don't show the input dialog */
static bool disableCancel;	 /* Disable cancel button on progress dialog */
static bool noConfirm;		 /* Don't ask for confirmation before writing */
static bool disableOpSel;	 /* Don't let user pick operation */
static bool disableBrowse;	 /* Don't let user change image file */
static bool disableDriveSel; /* Don't let the user change the drive */
static bool noSuccessMsg;	 /* Don't show the success message */

/*
 * Program Entry Point
 */
int WINAPI WinMain(HINSTANCE hInstance,		/* Current instance */
				   HINSTANCE hPrevInstance,	/* Previous running instance */
				   PSTR szCmdLine,
				   int iCmdShow) {

	long result;
	int  imagingResult;

	currentInstance = hInstance;

	processArgs();

	InitCommonControls();

	if (showAbout) {
		DialogBox(hInstance,	/* Handle to application instance */
				  MAKEINTRESOURCE(IDD_ABOUT),	/* Dialog Template */
				  NULL,		/* Parent */
				  reinterpret_cast<DLGPROC>(AboutDlgProc));	/* Dialog box procedure */
	}

	if (justGo)
		result = TRUE;
	else
		result = DialogBox(hInstance,
						   MAKEINTRESOURCE(IDD_IMG_DIALOG),
						   NULL,
						   reinterpret_cast<DLGPROC>(MainDlgProc));

	if (result == TRUE) {

		imagingResult = doImaging(selectedDrive,
								  image_mode,
								  noConfirm,
								  disableCancel,
								  szFileName,
								  hInstance);


		if (imagingResult == IMG_SUCCESS && !noSuccessMsg) {
			MessageBox(NULL,TEXT("Imaging Complete."),
				TEXT("Complete"), MB_OK | MB_ICONINFORMATION);
		} else if (imagingResult == IMG_CANCELED){
			MessageBox(NULL,TEXT("Imaging Canceled. Disk contents may be bad."),
				TEXT("Complete"), MB_OK | MB_ICONINFORMATION);
		}
		// Otherwise something else will have displayed an error message.
	} 

	return 0;
}

void processArgs() {

	LPTSTR *szArgList;
	int argCount;

	szArgList = CommandLineToArgv(GetCommandLine(),&argCount);
	if (szArgList == NULL) {
		MessageBox(NULL,TEXT("Unable to get command line arguments."),
				TEXT("Error"), MB_OK | MB_ICONWARNING);
		return;
	}

	setWrite = false;
	showAbout = false;
	memset(szFileName,0,MAX_FILE_LEN);
	setDrive = 0;
	selectedDrive = 'A';
	justGo = false;
	disableCancel = false;
	noConfirm = false;
	disableOpSel = false;
	disableBrowse = false;
	disableDriveSel = false;
	noSuccessMsg = false;

	for (int i = 1; i < argCount; i++) {
		// szArgList[i]
		if (lstrcmp(szArgList[i],TEXT("/about")) == 0) {
			// About switch supplied
			showAbout = true;
		} else if (lstrcmp(szArgList[i],TEXT("/w")) == 0) {
			// Set mode to WRITE
			setWrite = true;
			image_mode = IMGMODE_WRITE;
		} else if (lstrcmp(szArgList[i],TEXT("/r")) == 0) {
			// Set mode to READ
			image_mode = IMGMODE_READ;
			// read is the default for the GUI
		} else if (lstrcmp(szArgList[i],TEXT("/drive")) == 0) {
			// Set current drive to following entry

			i++;
			if (lstrcmp(szArgList[i],TEXT("b")) == 0) {
				setDrive = 1;
				selectedDrive = 'B';
			} else {
				setDrive = 0;
				selectedDrive = 'A';
			}

		} else if (lstrcmp(szArgList[i],TEXT("/go")) == 0) {
			// go
			justGo = true;
		} else if (lstrcmp(szArgList[i],TEXT("/nocancel")) == 0) {
			// no cancel
			disableCancel = true;
		} else if (lstrcmp(szArgList[i],TEXT("/noconfirm")) == 0) {
			// no confirm
			noConfirm = true;
		} else if (lstrcmp(szArgList[i],TEXT("/noopselect")) == 0) {
			// no op select
			disableOpSel = true;
		} else if (lstrcmp(szArgList[i],TEXT("/nodriveselect")) == 0) {
			// no op select
			disableDriveSel = true;
		} else if (lstrcmp(szArgList[i],TEXT("/nofilebrowse")) == 0) {
			// no file browse
			disableBrowse = true;
		} else if (lstrcmp(szArgList[i],TEXT("/nosuccessmsg")) == 0) {
			// Dont show the success message
			noSuccessMsg = true;
		} else {
			// filename
			lstrcpy(szFileName,szArgList[i]);
		}
	}

	LocalFree(szArgList);
}

/*
 * The dialog event processing function
 */
LRESULT CALLBACK MainDlgProc(HWND hwnd,
							 UINT iMsg,
							 WPARAM wParam,
							 LPARAM lParam) {

	DWORD dwResult;

	static DWORD aIds[] = {
		IDC_RB_READ_DISK,		IDH_OPERATION,
		IDC_RB_WRITE_DISK,		IDH_OPERATION,
		IDC_DRIVE_COMBO,		IDH_DRIVESEL,
		IDC_FILENAME,			IDH_FILESEL,
		IDC_BROWSE,				IDH_FILESEL,
		0,						0
	};


	switch(iMsg)
	{
	case WM_INITDIALOG:

		/*
		 * Check the Read Disk radio button by default
		 */
		if(setWrite) {
			SendDlgItemMessage(hwnd,
							   IDC_RB_WRITE_DISK,
							   BM_SETCHECK,
							   TRUE,
							   0);
			image_mode = IMGMODE_WRITE;
		} else {
			SendDlgItemMessage(hwnd,
							   IDC_RB_READ_DISK,
							   BM_SETCHECK,
							   TRUE,
							   0);
			image_mode = IMGMODE_READ;
		}

		populateDriveComboBox(hwnd);

		// Select the first drive by default
		SendDlgItemMessage(hwnd, /* dialog handle */
						  IDC_DRIVE_COMBO, /* control */
						  CB_SETCURSEL, /* message */
						  setDrive, /* first parameter */
						  0); /* second parameter */
		if (setDrive == 0)
			selectedDrive = 'A';
		else
			selectedDrive = 'B';

		dwResult = SendDlgItemMessage(hwnd,
								  IDC_FILENAME,
								  WM_SETTEXT,
								  0,
								  (LPARAM)szFileName);

		if (disableOpSel) {
			EnableWindow(GetDlgItem(hwnd,IDC_RB_WRITE_DISK),FALSE);
			EnableWindow(GetDlgItem(hwnd,IDC_RB_READ_DISK),FALSE);
		}

		if (disableDriveSel) {
			EnableWindow(GetDlgItem(hwnd,IDC_DRIVE_COMBO),FALSE);
		}

		if (disableBrowse) {
			EnableWindow(GetDlgItem(hwnd,IDC_FILENAME),FALSE);
			EnableWindow(GetDlgItem(hwnd,IDC_BROWSE),FALSE);
		}
		return TRUE;

	case WM_HELP:
		/* 
		 * Show context-sensitive help
		 */
        WinHelp((HWND)((LPHELPINFO) lParam)->hItemHandle, HELP_FILE, 
            HELP_WM_HELP, (DWORD) (LPSTR) aIds); 
		return TRUE;

	case WM_CONTEXTMENU:
		WinHelp((HWND) wParam, HELP_FILE, HELP_CONTEXTMENU, 
					(DWORD) (LPVOID) aIds); 
		return TRUE;

	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
			case IDOK:
				if (szFileName[0] == '\0') {
					MessageBox(NULL,TEXT("You must enter a filename."),
						TEXT("Error"), MB_OK | MB_ICONWARNING);
				} else {
					EndDialog(hwnd, TRUE);
				}
				return TRUE;
			case IDCANCEL:
				EndDialog(hwnd, FALSE);
				return TRUE;
			case IDC_HELPBTN:
				DialogBox(currentInstance,	/* Handle to application instance */
					   MAKEINTRESOURCE(IDD_ABOUT),	/* Dialog Template */
					   NULL,		/* Parent */
					   reinterpret_cast<DLGPROC>(AboutDlgProc));	/* Dialog box procedure */

				return TRUE;
			case IDC_BROWSE:
				browseForImageFile(hwnd);
				return TRUE;
			case IDC_RB_READ_DISK:
				/*
				 * First get the state of the check box then do something
				 * about it.
				 */
				dwResult = SendDlgItemMessage(hwnd, /* dialog handle */
											  IDC_RB_READ_DISK, /* control */
											  BM_GETSTATE, /* message */
											  0, /* first parameter */
											  0); /* second parameter */
				if (dwResult & BST_CHECKED) {
					image_mode = IMGMODE_READ;
				}

				/*
				 * Clear the filename text box to make sure the user doesn't
				 * accidentally overwrite anything. Only do it if setWrite
				 * is false though - if its true then this message was
				 * probably received because the write operation has just
				 * been preset.
				 */
				if (!setWrite) {
					clearFilenameTextBox(hwnd);
				} else {
					// Only ignore this message the first time.
					setWrite = false;
				}
				return TRUE;
			case IDC_RB_WRITE_DISK:
				/*
				 * First get the state of the check box then do something
				 * about it.
				 */
				dwResult = SendDlgItemMessage(hwnd, /* dialog handle */
											  IDC_RB_WRITE_DISK, /* control */
											  BM_GETSTATE, /* message */
											  0, /* first parameter */
											  0); /* second parameter */
				if (dwResult & BST_CHECKED) {
					image_mode = IMGMODE_WRITE;
				}

				/*
				 * Clear the filename text box to make sure the user doesn't
				 * accidentally overwrite anything. Only do it if setWrite
				 * is false though - if its true then this message was
				 * probably received because the write operation has just
				 * been preset.
				 */
				if (!setWrite) {
					clearFilenameTextBox(hwnd);
				} else {
					// Only ignore this message the first time.
					setWrite = false;
				}

				return TRUE;
			case IDC_DRIVE_COMBO:
				dwResult = SendDlgItemMessage(hwnd, /* dialog handle */
											  IDC_DRIVE_COMBO, /* control */
											  CB_GETCURSEL, /* message */
											  0, /* first parameter */
											  0); /* second parameter */
				if (dwResult == 0) {
					selectedDrive = 'A';
				} else if (dwResult == 1) {
					selectedDrive = 'B';
				} else {
					selectedDrive = ' ';
				}
				return TRUE;
			case IDC_FILENAME:
				dwResult = SendDlgItemMessage(hwnd,
											  IDC_FILENAME,
											  WM_GETTEXT,
											  (WPARAM)MAX_FILE_LEN,
											  (LPARAM)szFileName);
				return TRUE;
		}
		break;
	}

	return FALSE;
}

/*
 * Lets the user browse for a open or save filename
 */
void browseForImageFile(HWND hwndOwner) {
	OPENFILENAME ofn;

	DWORD dwResult;
	BOOL  bResult;

	/*
	 * Crazy things happen if we don't do this
	 */
	clearFilenameTextBox(hwndOwner);

	/*
	 * Setup the OPENFILENAME structure
	 */
	ofn.lStructSize			= sizeof(OPENFILENAME);
	ofn.hwndOwner			= hwndOwner;
	ofn.lpstrFilter			= szFilterSpec;
	ofn.lpstrCustomFilter	= NULL;
	ofn.nMaxCustFilter		= 0;
	ofn.nFilterIndex		= 1;
	ofn.lpstrFile			= szFileName;
	ofn.nMaxFile			= MAX_FILE_LEN;
	ofn.lpstrInitialDir		= NULL;
	ofn.lpstrFileTitle		= NULL;
	ofn.nMaxFileTitle		= MAX_FILE_LEN;
	ofn.lpstrDefExt			= TEXT("IMG");
	ofn.Flags				= OFN_HIDEREADONLY;
	ofn.lpfnHook			= NULL;
	ofn.lCustData			= 0;
	ofn.lpTemplateName		= 0;

	/*
	 * Get a filename from the user
	 */
	if(image_mode == IMGMODE_READ) {
		ofn.lpstrTitle = TEXT("Save Disk Image");
		ofn.Flags = ofn.Flags | OFN_OVERWRITEPROMPT;

		bResult = GetSaveFileName((LPOPENFILENAME)&ofn);

	} else if (image_mode == IMGMODE_WRITE) {
		ofn.lpstrTitle = TEXT("Open Disk Image");
		ofn.Flags = ofn.Flags | OFN_FILEMUSTEXIST;
		
		bResult = GetOpenFileName((LPOPENFILENAME)&ofn);
	} else {
		MessageBox(hwndOwner,
				   TEXT("Unexpected image mode."),
				   TEXT("ERROR"),
				   MB_OK | MB_ICONSTOP);
		return;
	}


	/* 
	 * There was an error or the user canceled 
	 */
	if (bResult == 0) {
		return;
	}

	/*
	 * Set the filename text boxes value
	 */
	dwResult = SendDlgItemMessage(hwndOwner,
								  IDC_FILENAME,
								  WM_SETTEXT,
								  0,
								  (LPARAM)szFileName);

}

/*
 * Clears the filename box
 */
void clearFilenameTextBox(HWND hwnd) {
	/*
	 * Clear the filename text box
	 */
	memset(szFileName,0,MAX_FILE_LEN);
	SendDlgItemMessage(hwnd,
					  IDC_FILENAME,
					  WM_SETTEXT,
					  0,
					  (LPARAM)szFileName);
}

/*
 * Populates the drive combo box
 */
void populateDriveComboBox(HWND hwnd) {
        TCHAR szItem[3] = TEXT("A:");
	SendDlgItemMessage(hwnd,
					   IDC_DRIVE_COMBO,
					   CB_ADDSTRING,
					   0,(LPARAM)szItem);

	szItem[0] = 'B';
	SendDlgItemMessage(hwnd,
					   IDC_DRIVE_COMBO,
					   CB_ADDSTRING,
					   0,(LPARAM)szItem);
}

/*
 * The about dialog event processing function
 */
LRESULT CALLBACK AboutDlgProc(HWND hwnd,
							 UINT iMsg,
							 WPARAM wParam,
							 LPARAM lParam) {

	switch(iMsg)
	{
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
			case IDOK:
				EndDialog(hwnd, TRUE);
				return TRUE;
		}
		break;
	}

	return FALSE;
}
