/*****************************************************************************
 * Application Name: WinVAM32 - Volume Access Module for 32bit Windows
 *        Copyright: (C) Copyright David Goodwin, 2010
 *				     Portions (C) 2001 John Elliott
 *          License: GNU GPL v2
 *****************************************************************************
	This file is part of "WinDIMG32".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 */

#include "geom.h"

int get_dosgeom(vam_diskgeom *geom, const char * bootsect, int bootsect_length) {
//int geom_detect::dosgeom(DiskGeometry *geom, const QByteArray *bootsect)

    /* This code derived from LibDsk (C) 2001  John Elliott
     */
        unsigned long lsmax;

        if (!geom || !bootsect) return VAMERR_BAD_POINTER;

		if (bootsect_length < 512) return VAMERR_NOT_ENOUGH_DATA;

		/* If the boot sector starts 0xE9 or 0xEB, it's DOS. If it starts with
		 * three zeroes, it's Atari.
		 *  In particular, we have to be careful not to try to identify a
		 * PCW 180k floppy, which starts 0x00 0x00 0x28 0x09 */

        if (bootsect[0] != (char)0xE9 && bootsect[0] != (char)0xEB)
        {
				/* However, the Mini Office distribution floppies for the Atari have only
				 * two zeroes. So if bytes 0B 0C 15 and 1B look something like a BPB,
				 * allow them. This should be sufficient to reject PCW diskettes */
                if (bootsect[0x0B] != (char)0    || bootsect[0x0C] != (char)2 ||
                    bootsect[0x15] != (char)0xF8 || bootsect[0x1B] != (char)0)
                {
                        if (bootsect[0] || bootsect[1] || bootsect[2])
                                return VAMERR_BAD_FORMAT;
                }
        }

        /* Reject fake DOS bootsectors created by 144FEAT */
        if (bootsect[511] == (char)144 || bootsect[511] == (char)72 || bootsect[511] == (char)12)
                return VAMERR_BAD_FORMAT;

        geom->sector_size   = ((int)bootsect[11]) + 256 * ((int)bootsect[12]);

        if ((geom->sector_size % 128) || (geom->sector_size == 0))
				/* Possible Apricot bootdisk if sector size is 0, or not a multiple of 128 */
				/* 		self->dg_secsize = 512; */
                return VAMERR_BAD_FORMAT;
        geom->sector_base   = 1;
        geom->heads        = ((int)bootsect[26]) + 256 * ((int)bootsect[27]);
        geom->sectors      = ((int)bootsect[24]) + 256 * ((int)bootsect[25]);
        if (!geom->heads || !geom->sectors) return VAMERR_BAD_FORMAT;
        lsmax = ((int)bootsect[19]) + 256 * ((int)bootsect[20]);
        lsmax /= geom->heads;
        lsmax /= geom->sectors;
        geom->cylinders = (int)lsmax;
        /* DOS boot sector doesn't store the data rate. We guess that if there are >12
         * sectors per track, it must have used high density to get them all in */
        geom->data_rate  = (geom->sectors >= 12) ? RATE_HD : RATE_SD;
        /* Similarly it doesn't store the gap lengths: */
        switch(geom->sectors)
        {
                case 8:  geom->rwGap = 0x2A; geom->fmtGap = 0x50; break;
                case 9:  geom->rwGap = 0x2A; geom->fmtGap = 0x52; break;
                case 10: geom->rwGap = 0x0C; geom->fmtGap = 0x17; break;
                case 15: geom->rwGap = 0x1B; geom->fmtGap = 0x50; break;
                case 18: geom->rwGap = 0x1B; geom->fmtGap = 0x50; break;
                default: geom->rwGap = 0x2A; geom->fmtGap = 0x52; break;
        }

        return VAMERR_SUCCESS;
}
