/*****************************************************************************
 * Application Name: WinVAM32 - Volume Access Module for 32bit Windows
 *        Copyright: (C) Copyright David Goodwin, 2010
 *          License: GNU GPL v2
 *****************************************************************************
	This file is part of "WinDIMG32".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 */

#include "vam_nt.h"

/*
 * Functions for Windows NT systems ONLY
 */

int write_data_nt(long sector, 
				  const char * data, 
				  int data_length, 
				  DWORD *dwBytesWritten) {

	DWORD dwVirtBufSize;
	LPVOID IoBuffer;

	dwVirtBufSize = diskgeom.sectors * diskgeom.sector_size;

	/* We only accept full tracks for now. */
	if (dwVirtBufSize > (DWORD)data_length)
		return VAMERR_NOT_ENOUGH_DATA;

	/* Allocate a buffer */
	IoBuffer = VirtualAlloc(NULL,dwVirtBufSize,MEM_COMMIT,PAGE_READWRITE);
	if (!IoBuffer) return VAMERR_MEM_ALLOC_FAILED;

	/* Copy data into buffer */
	memcpy(IoBuffer,data,dwVirtBufSize);

	/* Write buffer to disk */
	if (!WriteFile(hDrive,(LPVOID)IoBuffer,dwVirtBufSize, dwBytesWritten,NULL)) {
		VirtualFree(IoBuffer,0,MEM_RELEASE);
		return VAMERR_DEV_WRITE_FAILED;
	}

	/* Free buffer and return */
	VirtualFree(IoBuffer,0,MEM_RELEASE);
	return VAMERR_SUCCESS;
}

int read_data_nt(long track,
				 char* data,
				 int data_length,
				 int *length_read) {

	DWORD dwVirtBufSize, dwBytesRead;
	LPVOID IoBuffer;

	if (!geom_valid) {
		vam_read_diskgeom();
	}

	dwVirtBufSize = diskgeom.sector_size * diskgeom.sectors;

	/* Can't read more than we can store... */
	if (dwVirtBufSize > (DWORD)data_length) dwVirtBufSize = data_length;

	IoBuffer = VirtualAlloc(NULL,dwVirtBufSize,MEM_COMMIT,PAGE_READWRITE);

	if (!IoBuffer) {
		return VAMERR_MEM_ALLOC_FAILED;
	}

	if (!ReadFile(	hDrive,
					IoBuffer,
					dwVirtBufSize,
					&dwBytesRead,
					NULL)) {
		return VAMERR_DEV_READ_FAILED;
	}

	*length_read = dwVirtBufSize;
		
	memcpy(data,IoBuffer,dwVirtBufSize);

	return VAMERR_SUCCESS;
}

int read_diskgeom_nt() {
	LPVOID IoBuffer;
	DWORD dwBytesRead;
	DWORD newpos;
	int error_code;

	/* Allocate a buffer! */
	IoBuffer = VirtualAlloc(NULL,1024,MEM_COMMIT,PAGE_READWRITE);
	if (!IoBuffer) {
		return VAMERR_MEM_ALLOC_FAILED;
	}

	/* Attempt to read in the boot sector */
	if (!ReadFile(	hDrive,
					(LPVOID)IoBuffer,
					1024,
					&dwBytesRead,
					NULL)) {
		geom_valid = FALSE;
		return VAMERR_DEV_READ_FAILED;
	}

	/* Seek back to the start of the disk (so it is ready for reading later) */
	newpos = SetFilePointer(hDrive,			/* Device Handle */
							0,				/* Distance to move (lower 32 bits) */
							NULL,			/* Distance to move (upper 32 bits) */
							FILE_BEGIN);	/* Move method (where to start from */

	if (newpos != 0) {
		geom_valid = FALSE;
		return VAMERR_DEV_SEEK_FAILED;
	}

	error_code = vam_set_geometry_from_data((LPCSTR)IoBuffer, (int)dwBytesRead);

	VirtualFree(IoBuffer,0,MEM_RELEASE);

	if (error_code == VAMERR_SUCCESS)
		geom_valid = TRUE;

	return error_code;
}
