/*****************************************************************************
 * Application Name: WinVAM32 - Volume Access Module for 32bit Windows
 *        Copyright: (C) Copyright David Goodwin, 2010
 *          License: GNU GPL v2
 *****************************************************************************
	This file is part of "WinDIMG32".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 */

#ifndef WINVAM32_DLL_H
#define WINVAM32_DLL_H

#include <windows.h>

/* Library Export Stuff */
#ifdef __cplusplus
#define cppfudge "C"
#else
#define cppfudge
#endif

#ifdef BUILD_DLL
#define EXPORT __declspec(dllexport)
#else
#define EXPORT extern cppfudge __declspec(dllimport)
#endif

/* Error Codes */
#define VAMERR_SUCCESS			0
#define VAMERR_FAILURE			1
#define VAMERR_DRIVE_OPEN		2
#define VAMERR_OPEN_FAILED		3
#define VAMERR_LOCK_FAILED		4
#define VAMERR_UNLOK_FAILED		5
#define VAMERR_DISMT_FAILED		6
#define VAMERR_CLOSE_FAILED		7
#define VAMERR_BAD_POINTER		8
#define VAMERR_BAD_FORMAT		9
#define VAMERR_MEM_ALLOC_FAILED	10
#define VAMERR_DEV_WRITE_FAILED 11
#define VAMERR_DEV_READ_FAILED  12
#define VAMERR_DEV_SEEK_FAILED  13
#define VAMERR_NOT_ENOUGH_DATA  14

/* Disk Geometry Stuff */
typedef enum {
	RATE_HD,
	RATE_DD,
	RATE_SD,
	RATE_ED
} vam32_data_rate;

typedef struct {
	int cylinders;
	int heads;
	int sectors;
	int sector_size;
	int sector_base;
	vam32_data_rate data_rate;
	unsigned char rwGap;
	unsigned char fmtGap;
} vam_diskgeom;


/* Drive Control stuff */
EXPORT int vam_open_volume(char drive_letter);
EXPORT int vam_close_volume();
EXPORT BOOL vam_is_drive_open();

/* Disk Geometry related functions */
EXPORT void vam_set_geometry(vam_diskgeom dg);
EXPORT int vam_set_geometry_from_data(const char* bootsect, int bootsect_length);
EXPORT vam_diskgeom vam_get_geometry();
EXPORT int vam_read_diskgeom();

/* For reading and writing data */
EXPORT int vam_write_data(long track, 
						  const char * data,
						  int data_length,
						  DWORD *dwBytesWritten);
EXPORT int vam_read_data(long track, 
						 char * data, 
						 int data_length, 
						 int *length_read);

EXPORT int vam_get_version();

#endif /* WINVAM32_DLL_H */
